//---------------------------------------------------------------------------
#ifndef UartH
#define UartH
//---------------------------------------------------------------------------
#include <vcl\SysUtils.hpp>
#include <vcl\Controls.hpp>
#include <vcl\Classes.hpp>
#include <vcl\Forms.hpp>

//---------------------------------------------------------------------------
// Constants and enumerated types for TUart component
enum	TParity		{pNone=NOPARITY, pOdd=ODDPARITY, pEven=EVENPARITY, pMark=MARKPARITY, pSpace=SPACEPARITY};
enum	THandShake	{hsNone, hsXonXoff, hsCtsRts, hsDtrDsr};
enum	TStopBits	{sbOne=ONESTOPBIT, sbOneAndHalf=ONE5STOPBITS, sbTwo=TWOSTOPBITS};

//---------------------------------------------------------------------------
// Error codes
#define	UE_OPEN				1	/* could not open port						*/
#define UE_GETCOMMSTATE		2	/* GetCommState() failed					*/
#define UE_SETCOMMSTATE		3	/* SetCommState() failed					*/
#define UE_SETDTR			4	/* setting / clearing DTR failed			*/
#define UE_SETRTS			5	/* setting / clearing RTS failed			*/
#define	UE_SETBREAK			6	/* setting / clearing BREAK failed			*/

// line status codes
#define LS_CTS				MS_CTS_ON
#define	LS_DSR				MS_DSR_ON
#define	LS_RING				MS_RING_ON
#define LS_CD				MS_RLSD_ON
//---------------------------------------------------------------------------
// Custom events
typedef void __fastcall (__closure *TErrorEvent)(TObject *Sender, int ErrorCode, LPCSTR ErrMsg);
typedef void __fastcall (__closure *TRxNotifyEvent)(TObject *Sender, DWORD RxCount);
typedef void __fastcall (__closure *TTxNotifyEvent)(TObject *Sender, DWORD TxCount);
typedef void __fastcall (__closure *TLineEvent)(TObject *Sender, DWORD LineState);

//---------------------------------------------------------------------------
class TPollThread;			/* forward reference							*/
class PACKAGE TUart : public TComponent
{
private:
// Property data storage
	int			FPort;						/* Comms Port number			*/
	DWORD		FBaudRate;					/* Baudrate for port			*/
	TParity		FParity;  					/* Parity setting				*/
	THandShake	FHandShake;					/* Handshaking method			*/
	TStopBits	FStopBits;					/* number of stopbits			*/
	int			FDataBits;					/* number of databits to use	*/
	bool		FDTR;						/* Assert DTR on open or not	*/
	bool		FRTS;						/* Assert RTS on open or not	*/
	bool		FOpen;						/* TRUE if port is open			*/
	bool		FAutoOpen;					/* TRUE to open on startup		*/
	bool		FHandleErrors;				/* TRUE for us to handle errors	*/
	bool		FAbort;						/* TRUE to abort current comms	*/
	DWORD		FRxBuffSize;				/* Size of Receive buffer		*/
	DWORD		FTxBuffSize;				/* Size of Transmit buffer		*/
	DWORD		FRxNotifyLevel;				/* Number of bytes to trigger Rx*/

// Event handler storage...
	TErrorEvent	FErrorEvent;				/* OnError event handler		*/
	TRxNotifyEvent FRxNotifyEvent;			/* OnReceive event handler		*/
	TTxNotifyEvent FTxNotifyEvent;			/* OnTransmit event handler		*/
	TLineEvent	FLineEvent;					/* line status change event		*/

// Private data storage
	HANDLE		hComm;						/* handle for comms port		*/
	HANDLE		hWriteEvent;				/* event used for overlapped i/o*/
	HANDLE		hReadEvent;					/* event used for overlapped i/o*/
	HANDLE		hCommEvent;					/* event used for overlapped i/o*/
	OVERLAPPED	wOverlap;
	OVERLAPPED	rOverlap;
	OVERLAPPED	cOverlap;
	TPollThread	*pPoll;						/* ptr to thread for polling port*/
	DWORD		LastCount;					/* counter for notification		*/
	DCB			dcb;						/* holds comm settings			*/
	bool 		HaltUpdate;					/* TRUE to prevent updating		*/
	bool		HandleAssigned;				/* TRUE when using assigned handle*/
	bool		InWriting;					/* busy writing to comms port	*/
	bool		InReading;					/* busy reading from comms port	*/
	CRITICAL_SECTION	ReadSection;
	CRITICAL_SECTION	WriteSection;
	CRITICAL_SECTION	CommSection;

// Member routines not available to component users...
	void __fastcall ApplySettings();		/* opens/closes the comms port	*/
	bool __fastcall OpenPort();				/* opens the port, nothing else	*/
	void __fastcall ClosePort();			/* closes the port				*/
	void __fastcall UartError(int ErrCode, char *errmsg);/* handles uart errors	*/
	void __fastcall ClearErrors();			/* clears existing com errors	*/
	void __fastcall SetBufferSizes();		/* sets TX, RX buffer sizes		*/
	DWORD __fastcall WriteComm(BYTE *c, DWORD len, DWORD maxtime);
	DWORD __fastcall ReadComm(BYTE *b, DWORD len);

	virtual void __fastcall Loaded(void);	/* called after loading is complete	*/
protected:

public:

	__fastcall TUart(TComponent* Owner);
	__fastcall ~TUart();
	void __fastcall RxReady(DWORD count);
	void __fastcall LineChange();
// Design time property SET/GET routines...
	void __fastcall SetBaudRate(DWORD baud);
	void __fastcall SetHandShake(THandShake hs);
	void __fastcall SetStopBits(TStopBits sb);
	void __fastcall SetSerialPort(int port);
	void __fastcall SetDataBits(int db);
	void __fastcall SetParity(TParity par);
	void __fastcall SetDTR(bool setdtr);
	void __fastcall SetRTS(bool setrts);
	void __fastcall SetOpen(bool o);	/* opens / closes the port			*/
	void __fastcall SetRxBuff(DWORD size);
	void __fastcall SetTxBuff(DWORD size);
	void __fastcall SetRxNotifyLevel(DWORD size);
	bool __fastcall GetCTS();
	bool __fastcall GetDSR();
	bool __fastcall GetCD();
	bool __fastcall GetRING();
	DWORD __fastcall GetModemLines();
	DWORD __fastcall GetTxCount();
	DWORD __fastcall GetRxCount();

// methods...
	void __fastcall	SetAbort(bool a);		/* Aborts current comms			*/
	void __fastcall Flush();				/* empties queues				*/
	void __fastcall	AssignHandle(HANDLE h);	/* Assigns diffirent hComm handle*/
	void __fastcall ReleaseHandle();		/* releases previously assigned handle*/
	void __fastcall SetBreak(bool setbreak); /* turns BREAK ON/OFF			*/
// transmission methods
	bool __fastcall SendStr(char *str, DWORD maxtime = 5000);	/* Sends null-terminated string		*/
	bool __fastcall SendBuff(void *p, DWORD len, DWORD maxtime = 5000); /* sends buffer of bytes		*/
	bool __fastcall SendByte(BYTE c, DWORD maxtime = 5000);		/* sends 1 byte out the port		*/
	bool __fastcall SendString(String s, DWORD maxtime = 5000);
// reception methods
	DWORD __fastcall ReceiveStr(char *str, DWORD len, DWORD maxtime = 5000);
	DWORD __fastcall ReceiveBuff(void *buff, DWORD len, DWORD maxtime = 5000);
	bool __fastcall ReceiveByte(BYTE &c, DWORD maxtime = 5000);
	String __fastcall ReceiveString(DWORD len, DWORD maxtime = 5000);
	bool __fastcall WaitFor(BYTE c, DWORD maxtime = 5000);
// misc methods
	void __fastcall BeginUpdate();
	void __fastcall EndUpdate();

// Runtime properties...
	__property bool Open =			{read=FOpen,write=SetOpen,default=false};
	__property bool CTS =			{read=GetCTS};
	__property bool DSR =			{read=GetDSR};
	__property bool CD = 			{read=GetCD};
	__property bool	RING = 			{read=GetRING};
	__property DWORD ModemLines = 	{read=GetModemLines};
	__property DWORD RxCount =		{read=GetRxCount};
	__property DWORD TxCount =		{read=GetTxCount};
	__property bool Abort =			{read=FAbort,write=SetAbort};

__published:
// Design time properties...
	__property int		Port = 		{read=FPort,write=SetSerialPort,default=1};
	__property DWORD	Baudrate =	{read=FBaudRate,write=SetBaudRate,default=9600};
	__property int		Databits =	{read=FDataBits,write=SetDataBits,default=8};
	__property TStopBits Stopbits =	{read=FStopBits,write=SetStopBits,default=sbOne};
	__property TParity	Parity =	{read=FParity,write=SetParity,default=pNone};
	__property THandShake HandShaking={read=FHandShake,write=SetHandShake,default=hsNone};
	__property bool		DTR =		{read=FDTR,write=SetDTR,default=false};
	__property bool		RTS =		{read=FRTS,write=SetRTS,default=false};
	__property bool		AutoOpen =	{read=FAutoOpen,write=FAutoOpen,default=false};
	__property bool		HandleErrors={read=FHandleErrors,write=FHandleErrors,default=true};
	__property DWORD	RxBufferSize={read=FRxBuffSize,write=SetRxBuff,default=1024};
	__property DWORD	TxBufferSize={read=FTxBuffSize,write=SetTxBuff,default=1024};
//	__property DWORD	RxNotifyLevel={read=FRxNotifyLevel,write=SetRxNotifyLevel,default=1};
// Events
	__property TErrorEvent OnError=	{read=FErrorEvent,write=FErrorEvent};
	__property TRxNotifyEvent OnReceive={read=FRxNotifyEvent,write=FRxNotifyEvent};
	__property TTxNotifyEvent OnTransmit={read=FTxNotifyEvent,write=FTxNotifyEvent};
	__property TLineEvent	OnLineChange={read=FLineEvent,write=FLineEvent};
};
//---------------------------------------------------------------------------
// comms reading thread
class TPollThread : public TThread
{
private:
	HANDLE hComm;
	TUart *pUart;
protected:
	void __fastcall Execute();
	DWORD LastLevel;
	DWORD LineStatus;
	LPCRITICAL_SECTION pCommSection;
	LPOVERLAPPED pOverLap;
public:
	__fastcall TPollThread(TUart *uart, HANDLE ComFile, LPCRITICAL_SECTION cs, LPOVERLAPPED ol, bool CreateSuspended);
	void __fastcall SendEvent();
	void __fastcall SendLineStatus();
};

#endif

